clear all;close all;clc
data_dir=[pwd filesep];

%Add the function on this directory to the MATLAB path
%This is not permanent. This change is only valid for this session of
%MATLAB (will reset once MATLAB is restarted).
addpath(pwd)

if(exist('OCTAVE_VERSION'))
    more off %this seems necessary in order to get back the screen in Octave, but we have not tested this script on Octave yet.
end

%Check for previous files before starting test
data=dir(['asys.csv']);
if(~isempty(data))
    while(1)
        display(['Found previous answer sheet file in:' pwd])
        cont=input('Delete it (Y/N/Q)?','s');
        if(strcmp(cont,'Y') || strcmp(cont,'N') || strcmp(cont,'Q'))
            if(strcmp(cont,'Q'))
                display('Exiting script!!')
                return;
            end
            break;
        end
    end
    if(strcmp(cont,'Y'))
        display('Removing previous answer sheet.')
        delete(data.name);
    end
end

fprintf('Generating validation set, please wait...\n')
fid=fopen([data_dir 'ALARMS'],'r');
if(fid ~= -1)
    RECLIST=textscan(fid,'%s %s %d','Delimiter',',');
    fclose(fid);
else
    error('Could not open asys.csv for scoring. Exiting...')
end

matrix=zeros(16*125,122);
peaks_mat=zeros(50,122);
peaks_mat2=zeros(50,122);
peaks_mat3=zeros(50,122);

meta_mat=zeros(1,122);
RECORDS=RECLIST{1};
ALARMS=RECLIST{2};
GOLD=RECLIST{3};

%Specific to particular ALARM
index=find(ismember(ALARMS,'Asystole'));
RECORDS=RECORDS(index);
ALARMS=ALARMS(index);
GOLD=GOLD(index);

N=length(RECORDS);
results=zeros(N,1);

mark_5m = 125*60*5;

total_time=0;
for i=1:N
    recno = index(i);
    fprintf('**Scoring complete for: %d\n', i)

    % Name of file containing answers
    data = 'asys.csv';

    %Get all ECG, blood pressure and photoplethysmogram signals
    recordName=RECORDS{i};
    gold=GOLD(i);
    alarm_type=ALARMS{i};
    
    [~,signal,Fs,siginfo]=rdmat(recordName);
    alarmResult=1;

    % % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % % %%%%%%%%%%% Nadi 18/3/2015
    % % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    description=squeeze(struct2cell(siginfo));
    description=description(4,:);

    % Resample signal to 125Hz
    Fs=Fs(1);
    if Fs~=125
        signal=resample(signal,125,Fs);
        Fs=125;
    end

    % Adding ECG signals
    ecg_signal  = signal(:,1);
    ecg_signal  = medfilt1(ecg_signal);    
    ecg_snip    = ecg_signal(mark_5m-1999:mark_5m);
    
    matrix(:,i) = ecg_snip;
    Rpeaks      = QRSdet_Hilbert(ecg_signal',Fs);

    meta_mat(1,i)=gold;
    
    %Run WABP on the record, which by default will analyze the first ABP, ART, or BP signal
    N=[];
    N0=[];
    abp_ind=get_index(description,'ABP');
    ann_abp=[];
    features=[];
    BEATQ=[];
    R=[];
    if(~isempty(abp_ind))
       ann_abp=wabp(signal(:,abp_ind),0,1);
       % Analyze the signal quality index of ABP using jSQI
       if length(ann_abp)>=3 % at least 3 abp beats detected
            [features] = abpfeature(signal(:,abp_ind),ann_abp);
            [BEATQ R] = jSQI(features, ann_abp, signal(:,abp_ind));
       end
    end

    %Run WABP on the record of 'PLETH' to analyze photoplethysmogram signal
    ppg_ind=get_index(description,'PLETH');
    ann_ppg=[];
    if (~isempty(ppg_ind))
        y=quantile(signal(:,ppg_ind),[0.05,0.5,0.95]);
        ann_ppg=wabp(signal(:,ppg_ind),0,(y(3)-y(1))/120);
        % Analyze the signal quality index of PPG 
        if ~isempty(ann_ppg)
            [psqi]=ppgSQI(signal(:,ppg_ind),ann_ppg);
        end
    end

    %Make decisions

    % set valid data segment for decision making, 16s before the alarm
    N_d=Fs*5*60; % alarm position
    N0_d=N_d-Fs*16+1; % 16s before the alarm

    % select the beats in the segment
    n_abp_beats=intersect(find(ann_abp>=N0_d),find(ann_abp<=N_d));
    n_ppg_beats=intersect(find(ann_ppg>=N0_d),find(ann_ppg<=N_d));
    %Added ECG II
    n_ECG2_beats=intersect(find(Rpeaks>=N0_d),find(Rpeaks<=N_d));
    %Added ECG V
    % n_ECG5_beats=intersect(find(Rpeaks5>=N0_d),find(Rpeaks5<=N_d));

    hr_max_abp=NaN;
    max_rr_abp=NaN;
    max_rr_abp=NaN;
    hr_max_ppg=NaN;
    max_rr_ppg=NaN;
    hr_max_ECG2=NaN;
    max_rr_ECG2=NaN;
    no_p_wave_cnt = 0;

    % calculate the heart rate
    if length(n_abp_beats)>=2
        hr_max_abp=60*Fs/min(diff(ann_abp(n_abp_beats)));
        max_rr_abp=max(diff(ann_abp(n_abp_beats)))/Fs;
    end
    if length(n_ppg_beats)>=2
        hr_max_ppg=60*Fs/min(diff(ann_ppg(n_ppg_beats)));
        max_rr_ppg=max(diff(ann_ppg(n_ppg_beats)))/Fs;
    end
    %Added ECG II
    if length(n_ECG2_beats)>=2
        hr_max_ECG2=60*Fs/min(diff(Rpeaks(n_ECG2_beats)));
        max_rr_ECG2=max(diff(Rpeaks(n_ECG2_beats)))/Fs;
    end

    peaks_mat(1:length(Rpeaks(n_ECG2_beats)),i) = Rpeaks(n_ECG2_beats);
    peaks_mat2(1:length(ann_abp(n_abp_beats)),i) = ann_abp(n_abp_beats);
    peaks_mat3(1:length(ann_ppg(n_ppg_beats)),i) = ann_ppg(n_ppg_beats);
    
    %rough P-wave detection for VTach
    l_ECG2_window=Fs*0.2; %Search in window of 200 ms
    pwave_th = 0.1;
    
%     %First 10 beats
%     
%     if length(n_ECG2_beats)>=10
%         for i=1:10
%             ecg_sig     = ecg_signal(Rpeaks(n_ECG2_beats(i))-l_ECG2_window:Rpeaks(n_ECG2_beats(i)));
%             ecg_sig_dt  = diff(ecg_sig);
%             ecg_sig_dt2 = diff(ecg_sig_dt);
%             ecg_p_idx = find(ecg_sig_dt==0);
%             ecg_p_idx(ecg_p_idx==1)=[];
%             [mn, mn_idx] = min(ecg_sig);
%             ecg_p_idx(ecg_p_idx>=mn_idx)=[];
%             cnt = 0;
% 
%             if ~isempty(ecg_p_idx) 
%                 sig_val = ecg_sig(ecg_p_idx);
%                 mx_sig_val = max(sig_val);
%                 if  mx_sig_val - mn < pwave_th %No p-wave detected
%                     cnt = 1;
%                 end
%             else
%                 cnt = 1;
%             end
%             no_p_wave_cnt = no_p_wave_cnt + cnt;
%         end
%     end    
%     %Added ECG V
%     % if length(n_ECG5_beats)>=2
%     %     hr_max_ECG5=60*Fs/min(diff(Rpeaks5(n_ECG5_beats)));
%     %     max_rr_ECG5=max(diff(Rpeaks5(n_ECG5_beats)))/Fs;
%     % end
% 
%     % calculate low heart rate of 5 consecutive beats for Bradycardia
%     low_hr_abp=NaN;
%     low_hr_ppg=NaN;
%     low_hr_ECG2=NaN;
%     % low_hr_ECG5=NaN;
%     if length(n_abp_beats>=5)
%         for i=1:length(n_abp_beats)-4
%             low_hr_abp(i)=60*Fs/((ann_abp(n_abp_beats(i+4))-ann_abp(n_abp_beats(i)))/4);
%         end
%     end
%     low_hr_abp=min(low_hr_abp);
%     if length(n_ppg_beats>=5)
%         for i=1:length(n_ppg_beats)-4
%             low_hr_ppg(i)=60*Fs/((ann_ppg(n_ppg_beats(i+4))-ann_ppg(n_ppg_beats(i)))/4);
%         end
%     end
%     low_hr_ppg=min(low_hr_ppg);
%     % Adding ECG II        
%     if length(n_ECG2_beats>=5)
%         for i=1:length(n_ECG2_beats)-4
%             low_hr_ECG2(i)=60*Fs/((Rpeaks(n_ECG2_beats(i+4))-Rpeaks(n_ECG2_beats(i)))/4);
%         end
%     end
%     low_hr_ECG2=min(low_hr_ECG2);
%     % Adding ECG V        
%     % if length(n_ECG5_beats>=5)
%     %     for i=1:length(n_ECG5_beats)-4
%     %         low_hr_ECG5(i)=60*Fs/((Rpeaks5(n_ECG5_beats(i+4))-Rpeaks5(n_ECG5_beats(i)))/4);
%     %     end
%     % end
%     % low_hr_ECG5=min(low_hr_ECG5);
% 
% 
%     % calculate high heart reate of 17 consecutive beats for Tachycardia
%     high_hr_abp=NaN;
%     high_hr_ppg=NaN;
%     high_hr_ECG2=NaN;
%     % high_hr_ECG5=NaN;
%     if length(n_abp_beats>=17)
%         for i=1:length(n_abp_beats)-16
%             high_hr_abp(i)=60*Fs/((ann_abp(n_abp_beats(i+16))-ann_abp(n_abp_beats(i)))/16);
%         end
%     end
%     high_hr_abp=max(high_hr_abp);
%     if length(n_ppg_beats>=17)
%         for i=1:length(n_ppg_beats)-16
%             high_hr_ppg(i)=60*Fs/((ann_ppg(n_ppg_beats(i+16))-ann_ppg(n_ppg_beats(i)))/16);
%         end
%     end
%     high_hr_ppg=max(high_hr_ppg);
%     % Adding ECG II
%     if length(n_ECG2_beats>=17)
%         for i=1:length(n_ECG2_beats)-16
%             high_hr_ECG2(i)=60*Fs/((Rpeaks(n_ECG2_beats(i+16))-Rpeaks(n_ECG2_beats(i)))/16);
%         end
%     end
%     high_hr_ECG2=max(high_hr_ECG2);
%     % % Adding ECG V
%     % if length(n_ECG5_beats>=17)
%     %     for i=1:length(n_ECG5_beats)-16
%     %         high_hr_ECG5(i)=60*Fs/((Rpeaks5(n_ECG5_beats(i+16))-Rpeaks5(n_ECG5_beats(i)))/16);
%     %     end
%     % end
%     % high_hr_ECG5=max(high_hr_ECG5);
% 
%     % calculate the signal quality index
%     if ~isempty(ann_abp)
%         abpsqi=1-sum(sum(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:)))/numel(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:));
%     else
%         abpsqi=0;
%     end
%     if ~isempty(ann_ppg)
%         ppgsqi=mean(psqi(intersect(n_ppg_beats,1:length(psqi))));
%     else
%         ppgsqi=0;
%     end
% 
%     % Adding ECG II & V
%     % Just adding a condition of max_rr_ECG2<6: Meaning that if the RR interval
%     % has been greater than 6 seconds, assume the ECG signal as low quality and
%     % do not consider it in arrhythmia detection.
% 
%     % SQI threshold
%     sqi_th = 0.95;
% 
%     % Alarm threshold (seconds)
%     ASY_th = 4;
%     BRA_th = 40;
%     TAC_th = 140;
%     VTA_th = 100;
%     VFB_th = 250;
%     VTA_beats_mx = 5;
%     tolerance = 10; % tolerance = 5 bmp
% 
%     % Write result to answers.txt
%     fid = fopen(data, 'a');
%     if (fid == -1)
%         error('Could not open answer file');
%     end
% 
%     fprintf(fid, '%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\r\n', recno, gold, abpsqi, hr_max_abp, max_rr_abp, ppgsqi, hr_max_ppg, max_rr_ppg, hr_max_ECG2, max_rr_ECG2, no_p_wave_cnt);
%     fclose(fid);
    
end

csvwrite('ecg_data_asys.csv',matrix);
csvwrite('ecg_peaks_asys.csv',peaks_mat);
csvwrite('ecg_peaks_asys2.csv',peaks_mat2);
csvwrite('ecg_peaks_asys3.csv',peaks_mat3);
csvwrite('ecg_meta_asys.csv',meta_mat);

fprintf(['**Scoring complete.\n'])